"""
Starts the edurov-web version.
"""

import signal

import Pyro4

from edurov import WebMethod
from edurov.arduino import get_serial_connection, send_arduino, \
    receive_arduino, valid_arduino_string
from edurov.utils import detect_pi

if detect_pi():
    from sense_hat import SenseHat


def arduino(debug=False):
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    states = [0, 0, 0, 0]
    lastState = '0000'
    if not debug:
        ser = get_serial_connection()
    with Pyro4.Proxy("PYRONAME:KeyManager") as keys:
        with Pyro4.Proxy("PYRONAME:ROVSyncer") as rov:
            keys.set_mode(key='l', mode='toggle')
            while rov.run:
                dic = keys.qweasd_dict
                if dic['w']:
                    states[0] = 1
                elif dic['s']:
                    states[0] = 2
                else:
                    states[0] = 0

                if dic['a']:
                    states[1] = 1
                elif dic['q']:
                    states[1] = 2
                else:
                    states[1] = 0

                if dic['e']:
                    states[2] = 2
                elif dic['d']:
                    states[2] = 1
                else:
                    states[2] = 0
                light_state = int(keys.state('l'))
                states[3] = light_state

                state = ''.join([str(n) for n in states])
                if state != lastState:
                    lastState = state
                    if not debug and ser:
                        send_arduino(msg=state, serial_connection=ser)
                    else:
                        print(state)
                if not debug and ser:
                    arduino_string = receive_arduino(serial_connection=ser)
                    if valid_arduino_string(arduino_string):
                        v1, v2, v3 = arduino_string.split(':')

                        rov.sensor = {
                            'tempWater': float(v1),
                            'pressureWater': float(v2),
                            'batteryVoltage': float(v3),
                            'light': light_state
                        }


def senser(debug=False):
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    sense = SenseHat()
    with Pyro4.Proxy("PYRONAME:ROVSyncer") as rov:
        while rov.run:
            orientation = sense.get_orientation()
            rov.sensor = {'temp': sense.get_temperature(),
                          'pressure': sense.get_pressure(),
                          'humidity': sense.get_humidity(),
                          'pitch': orientation['pitch'],
                          'roll': orientation['roll'] + 180,
                          'yaw': orientation['yaw']}


def main(video_resolution='1024x768', fps=30, server_port=8000, debug=False):
    web_method = WebMethod(
        video_resolution=video_resolution,
        fps=fps,
        server_port=server_port,
        debug=debug,
        runtime_functions=[arduino, senser]
    )
    web_method.serve()


if __name__ == '__main__':
    main()



import subprocess
import time
from multiprocessing import Process

import Pyro4

from edurov.sync import start_sync_classes
from edurov.utils import warning, preexec_function
from edurov.web.servers import start_http_server


class WebMethod(object):
    def __init__(self, video_resolution='1024x768', fps=30, server_port=8000,
                 debug=False, runtime_functions=None):
        if callable(runtime_functions):
            runtime_functions = [runtime_functions]
        elif isinstance(runtime_functions, list):
            for f in runtime_functions:
                if not callable(f):
                    warning('Parameter runtime_functions has to be a function '
                            'or a list of functions, not {}'.format(type(f)))
        else:
            warning('Parameter runtime_functions has to be a function '
                    'or a list of functions, not {}'
                    .format(type(runtime_functions)))

        self.res = video_resolution
        self.fps = fps
        self.server_port = server_port
        self.debug = debug
        self.run_funcs = runtime_functions

    def serve(self, timeout=None):
        start = time.time()
        name_server = subprocess.Popen('pyro4-ns', shell=False,
                                       preexec_fn=preexec_function)
        time.sleep(2)
        pyro_classes = Process(target=start_sync_classes)
        pyro_classes.start()
        time.sleep(4)
        web_server = Process(target=start_http_server,
                             args=(
                             self.res, self.fps, self.server_port, self.debug))
        web_server.start()
        processes = []
        for f in self.run_funcs:
            p = Process(target=f)
            p.start()
            processes.append(p)

        with Pyro4.Proxy("PYRONAME:ROVSyncer") as rov:
            try:
                while rov.run:
                    if timeout:
                        if time.time()-start >= timeout:
                            break
            except KeyboardInterrupt:
                pass
            finally:
                print('Shutting down')
                web_server.terminate()
                rov.run = False
                for p in processes:
                    p.join(3)
                pyro_classes.terminate()
                name_server.terminate()


class KeyConnection(object):
    def __init__(self):
        pass


class RovConnection(object):
    def __init__(self):
        pass
